import requests
from timekit.components.base import *
__all__ = ["Resource"]


class Resource(APIBaseMethod):
    def __init__(self, app_token):
        APIBaseMethod.__init__(self, app_token)
        self.url = "{}/resources".format(self.url)

    def retrieve(self, id, dynamic_includes=[]) -> Response:
        includes = ''
        if dynamic_includes != []:
            includes = ','.join(dynamic_includes)
        response = requests.get("{}/{}?include={}".format(
            self.url, id, includes),
                                json=None,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def availability_constraints(self, id, data) -> Response:
        response = requests.put("{}/{}".format(self.url, id),
                                json=data,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)



import requests
from timekit.components.base import *
__all__ = ["Booking"]


class Group(APIBaseMethod):
    def __init__(self, app_token):
        APIBaseMethod.__init__(self, app_token)
        self.url = "{}/bookings/groups".format(self.url)


class Booking(APIBaseMethod):
    def __init__(self, app_token):
        APIBaseMethod.__init__(self, app_token)
        self.url = "{}/bookings".format(self.url)
        self.groups = Group(app_token)

    def create(self, data, dynamic_includes=[]) -> Response:
        """
        data: json object, 
        dynamic_includes: list
        for more information refer https://developers.timekit.io/reference#bookings
        """
        includes = ""
        if dynamic_includes != [] and type(dynamic_includes) is list:
            includes = ",".join(dynamic_includes)
        request_url = "{}?include={}".format(self.url, includes)
        response = requests.post(request_url,
                                 json=data,
                                 headers=self.headers,
                                 auth=self.auth)
        return handle_response(response)

    def list(self,
             limit=50,
             page=1,
             dynamic_includes=[],
             search="",
             order_by=None,
             sorted_by=None) -> Response:
        """
        limit: integer (1000 max), 
        page: integer,
        dynamic_includes: list, 
        search: string, 
        order_by: string, 
        sorted_by: string
        for more information refer https://developers.timekit.io/reference#bookings-2
        """
        includes = ""
        if dynamic_includes != [] and type(dynamic_includes) is list:
            includes = ",".join(dynamic_includes)
        request_url = "{}?limit={}&page={}include={}".format(
            self.url, limit, page, includes)
        if search:
            request_url = '{}&search={}'.format(request_url, search)
        if order_by:
            request_url = '{}&orderBy={}'.format(request_url, order_by)
        if sorted_by:
            request_url = '{}&sortedBy={}'.format(request_url, sorted_by)
        response = requests.get(request_url,
                                json=None,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def update_state(self, id, action) -> Response:
        """
        id: string(id generated by timekit)
        action: string(confirm|decline|cancel|cancel_by_owner)
        for more information refer https://developers.timekit.io/reference#bookingsidaction
        """
        if action and action not in [
                'confirm', 'decline', 'cancel', 'cancel_by_owner'
        ]:
            raise Exception("function argument action is invalid!")
        response = requests.put("{}/{}/{}".format(self.url, id, action),
                                auth=self.auth,
                                headers=self.headers)
        return handle_response(response)

    def update_meta(self, id, data) -> Response:
        """
        id: string(id generated by timekit)
        data: json object
        for more information refer https://developers.timekit.io/reference#update-booking-meta-data
        """
        response = requests.put("{}/{}".format(self.url, id),
                                json=data,
                                auth=self.auth,
                                headers=self.headers)
        return handle_response(response)

    def create_in_bulk(self, data) -> Response:
        """
        data: json object containing multiple booking details
        for more information refer https://developers.timekit.io/reference#bookingsbulk
        """
        response = requests.post("{}".format(self.url),
                                 json=data,
                                 auth=self.auth,
                                 headers=self.headers)
        return handle_response(response)

    def update_in_bulk(self, data) -> Response:
        """
        data: json object containing multiple booking details
        for more information refer https://developers.timekit.io/reference#bookingsbulk-1
        """
        response = requests.put("{}".format(self.url),
                                json=data,
                                auth=self.auth,
                                headers=self.headers)
        return handle_response(response)


import requests
from timekit.components.base import *
__all__ = ["App"]


class App(APIBaseMethod):
    def __init__(self, app_token):
        APIBaseMethod.__init__(self, app_token)
        self.url = "{}/app".format(self.url)

    def get(self) -> Response:
        request_url = "{}".format(self.url)
        response = requests.get(request_url,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def invite(self, data) -> Response:
        request_url = "{}/invite".format(self.url)
        response = requests.post(request_url,
                                 json=data,
                                 headers=self.headers,
                                 auth=self.auth)
        return handle_response(response)



import requests
__all__ = ["APIBaseMethod", "Response", "handle_response"]

BASE_URL_V2 = "https://api.timekit.io/v2"


class Response:
    def __init__(self, data, status):
        self.data = data
        self.status = status


def handle_response(response):
    if response.status_code in range(200, 206):
        try:
            return Response(response.json(), response.status_code)
        except:
            return Response({}, response.status_code)
    else:
        try:
            return Response(response.json(), response.status_code)
        except:
            if response.status_code == 401:
                return Response({"error": "Invalid credentials supplied"},
                                response.status_code)
            elif response.status_code == 422:
                return Response(
                    {
                        "error":
                        "A POST data JSON key or alike is malformed or missing"
                    }, response.status_code)
            else:
                return Response({"error": "response is not json serializable"},
                                response.status_code)


class APIBaseMethod:
    def __init__(self, app_token):
        self.headers = {'Content-Type': 'application/json'}
        self.auth = requests.auth.HTTPBasicAuth('', app_token)
        self.url = BASE_URL_V2

    def list(self, limit=50, page=1, search="") -> Response:
        """
        limit: integer(1000 max),
        page: integer,
        search: string,
        Refer timekit API documentation for search
        https://developers.timekit.io/reference#getting-started
        """
        request_url = '{}?limit={}&page={}&search={}'.format(
            self.url, limit, page, search)
        response = requests.get(request_url,
                                json=None,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def create(self, data) -> Response:
        """
        data: json object
        Refer timekit API documentation for json data structure
        https://developers.timekit.io/reference#getting-started
        """
        response = requests.post(self.url,
                                 json=data,
                                 headers=self.headers,
                                 auth=self.auth)
        return handle_response(response)

    def retrieve(self, id, data=None) -> Response:
        """
        id: string(id generated by timekit)
        """
        response = requests.get("{}/{}".format(self.url, id),
                                json=None,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def update(self, id, data) -> Response:
        """
        id: string(id generated by timekit),
        data: json object
        Refer timekit API documentation for json data structure
        https://developers.timekit.io/reference#getting-started
        """
        response = requests.put("{}/{}".format(self.url, id),
                                json=data,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def partial_update(self, id, data) -> Response:
        """
        id: string(id generated by timekit),
        data: json object
        Refer timekit API documentation for json data structure
        https://developers.timekit.io/reference#getting-started
        """
        response = requests.patch("{}/{}".format(self.url, id),
                                  json=data,
                                  headers=self.headers,
                                  auth=self.auth)
        return handle_response(response)

    def delete(self, id) -> Response:
        """
        id: string(id generated by timekit)
        """
        response = requests.delete("{}/{}".format(self.url, id),
                                   json=None,
                                   headers=self.headers,
                                   auth=self.auth)
        return handle_response(response)



from __future__ import print_function, unicode_literals

from setuptools import setup
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))
with open('README.md', 'r') as fh:
    long_description = fh.read()


def read(file_paths, default=""):
    # intentionally *not* adding an encoding option to open
    try:
        with codecs.open(os.path.join(here, *file_paths), "r") as fh:
            return fh.read()
    except Exception:
        return default


def find_version(file_paths):
    version_file = read(file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='python_timekit',
    version=find_version(["timekit", "__init__.py"]),
    url='https://github.com/Srijan-Ramavat/python-timekit',
    author="Srijan Ramavat",
    author_email="srijan.ramavat@gmail.com",
    description=
    'An un-official package to integrate Timekit APIs with your project',
    packages=['timekit', 'timekit.components'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Topic :: Internet",
        "Topic :: Office/Business",
        "Topic :: Software Development :: Libraries",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "requests==2.24.0",
    ],
)


import requests
from timekit.components.base import *
__all__ = ["Project"]


class Project(APIBaseMethod):
    def __init__(self, app_token):
        APIBaseMethod.__init__(self, app_token)
        self.url = "{}/projects".format(self.url)

    def add_resources(self, id, data) -> Response:
        response = requests.post("{}/{}/resources".format(self.url, id),
                                 json=data,
                                 headers=self.headers,
                                 auth=self.auth)
        return handle_response(response)

    def set_resources(self, id, data) -> Response:
        response = requests.put("{}/{}/resources".format(self.url, id),
                                json=data,
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def remove_resources(self, id, resource_id) -> Response:
        response = requests.put("{}/{}/resources/{}".format(
            self.url, id, resource_id),
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)

    def get_resources(self, id) -> Response:
        response = requests.get("{}/{}/resources".format(self.url, id),
                                headers=self.headers,
                                auth=self.auth)
        return handle_response(response)



"""Timekit.io REST API Python Client"""
from __future__ import absolute_import, unicode_literals
from timekit import components


class TimekitAPI:
    def __init__(self, app_token):
        self.apps = components.apps.App(app_token)
        self.bookings = components.bookings.Booking(app_token)
        self.projects = components.projects.Project(app_token)
        self.resources = components.resources.Resource(app_token)
